# sb::Admin::User
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==

# ======================================
package sb::Admin::User;
use strict;
use sb::Admin::List ();
use base qw( sb::Admin::List );
use vars qw( $VERSION );
$VERSION = '0.04';
# 0.04 [2009/02/09] modified title_name to make it clearer
# 0.03 [2008/12/07] modified _update to update $self->users as well
# 0.02 [2007/11/05] added title_name
# 0.01 [2007/09/15] ported from Serene Bach 2.*
# 0.00 [2005/02/02] generated
# --------------------------------------
# // defines
sub TEMPLATE     (){ 'user.html' };
sub MAX_LEVEL    (){ 2 };
sub LIST_COLUMN  (){ 8 };
sub DENIED_CHECK (){ '-' };
# --------------------------------------
# // public functions - for sub class
sub check_user
{
	my $self = shift;
	my %param = (
		'name'       => undef,
		'pass'       => undef,
		'conf'       => undef,
		'check_pass' => 1,
		'check_name' => 1,
		@_
	);
	if ($param{'check_pass'})
	{
		return sb->string('Invalid characters in password.') if ($param{'pass'} !~ /^\w+$/);
		return sb->string('Password confirmation is incorrect.') if ($param{'pass'} ne $param{'conf'});
	}
	if ($param{'check_name'})
	{
		my $check = sb::Data->load('User','cond'=>{'name'=>$param{'name'}});
		return sb->string('Invalid characters in user name.') if ($param{'name'} !~ /^[a-zA-Z0-9_\-\.]+$/);
		return sb->string('This user name has already been used.') if ($check);
	}
	return;
}
# --------------------------------------
# // override base class
sub title_name
{
	my $self = shift;
	my $title = $self->SUPER::title_name();
	return $title if ($title);
	return sb->string(' &gt; Users') if ($self->mode eq 'user');
	return;
}
# --------------------------------------
# // public functions
sub callback
{
	my $self = shift;
	$self->{'user_error'} = undef;
	return ($self->regi)
		? $self->_update(@_)
		: $self->_display(@_);
}
# --------------------------------------
# // private functions
sub _update
{
	my $self = shift;
	my $msg = undef;
	if ($self->cgi->value('__regi') eq 'add')
	{ # create new user
		$self->{'user_error'} = $self->check_user(
			'name' => $self->cgi->value('user_name'),
			'pass' => $self->cgi->value('user_pass'),
			'conf' => $self->cgi->value('user_passconf'),
		);
		if (!$self->{'user_error'})
		{
			my $level = int($self->cgi->value('user_stat'));
			my %elem = (
				'name' => $self->cgi->value('user_name'),
				'mail' => sb::Text->new($self->cgi->value('user_mail'))->entitized(),
				'real' => sb::Text->new($self->cgi->value('user_real'))->entitized(),
				'stat' => ($level <= 0 or $level > MAX_LEVEL) ? MAX_LEVEL : $level,
			);
			my $new = sb::Data->add('User',%elem);
			$new->pass($self->cgi->value('user_pass'));
			$self->{'users'}->{$new->id} = $new;
			sb::Data->update($new);
			$msg = sb->string('Your request has been processed.');
		}
	}
	elsif ( $self->cgi->value('del') ne '' )
	{ # delete the selected user
		my @sels = split("\0",$self->cgi->value('sel'));
		my @users = sb::Data->load('User','cond'=>{'id'=>\@sels});
		foreach my $user (@users)
		{
			next if ($user->id eq '0');
			delete $self->{'users'}->{$user->id};
			$user->erase;
		}
		sb::Data->update(@users) if (@users);
		$msg = sprintf(sb->string('Deleted %d item(s).<br />'),$#users + 1);
	}
	else
	{ # change the order of user list
		my @ids = split("\0",$self->cgi->value('sel_id'));
		my @users = sb::Data->load('User','cond'=>{'id'=>\@ids});
		my $order = undef;
		foreach my $user (@users)
		{
			next if ($self->cgi->value('dn' . $user->id) eq '' and $self->cgi->value('up' . $user->id) eq '');
			$order = $user;
			last;
		}
		if ($order)
		{
			@users = $self->change_order(
				'data'      => [ sb::Data->load('User','sort'=>'order','order'=>1) ],
				'target'    => $order,
				'direction' => ($self->cgi->value('up' . $order->id) ne '') ? +1 : -1,
			);
			if ( @users )
			{
				map { $self->{'users'}->{$_->id} = $_ } @users;
				sb::Data->update(@users);
			}
			$msg = sb->string('Modified.<br />');
		}
	}
	$self->build_list('user_list');
	return $self->_display('message'=>$msg);
}
sub _display
{
	my $self = shift;
	my %param = (
		'message' => undef,
		'setup'   => $self->setup_list(),
		@_
	);
	my $temp = sb::Template->new($self->load_template('file'=>TEMPLATE));
	my $page = $self->get_current_page();
	my @users = sb::Data->load('User',
		'sort'  => 'order',
		'num'   => $param{'setup'}->{'dispnum'},
		'bgn'   => $page * $param{'setup'}->{'dispnum'},
		'order' => 1,
	);
	my $matched = sb::Data->matched;
	$temp->num(0);
	$temp->tag('page_now'=>$page);
	$temp->tag('sb_list_page'=>$self->display_pagelink(
			'mode'    => 'user',
			'column'  => LIST_COLUMN,
			'all'     => $matched,
			'printed' => $#users + 1,
			'num'     => $param{'setup'}->{'dispnum'},
			'params'  => ['dispnum'],
		)
	);
	$self->dispnum_selector(
		'cms'  => $temp,
		'now'  => $param{'setup'}->{'dispnum'},
	);
	$self->listmain(
		'template' => $temp,
		'block'    => 'sb_user_list',
		'objects'  => \@users,
		'tags'     => {
			'sb_userlist_id'   => 'id',
			'sb_userlist_name' => 'name',
			'sb_userlist_real' => 'real',
			'sb_userlist_mail' => 'mail',
			'sb_userlist_stat' => \&_display_status_mark,
			'sb_userlist_del'  => \&_display_checkbox,
		},
	);
	$self->order_marker(
		'template' => $temp,
		'objects'  => \@users,
		'matched'  => $matched,
		'tag'      => 'sb_userlist_order',
		'page'     => $page,
		'number'   => $param{'setup'}->{'dispnum'},
	);
	if ($self->{'user_error'})
	{ # we keep the previous input when creation of a user is failed.
		$temp->num(0);
		foreach my $key ('name','real','mail')
		{
			next if ($self->cgi->value('user_' . $key) eq '');
			my $value = sb::Text->new($self->cgi->value('user_' . $key));
			$temp->tag('sb_user_' . $key => $value->entitized());
		}
		$param{'message'} = $self->{'user_error'};
	}
	$self->common_parts($temp);
	$self->set_message($temp,$param{'message'});
	return $self->set_main($temp->output)
}
# --------------------------------------
# // private functions - list handlers
sub _display_checkbox
{
	my $self = shift;
	my $obj  = shift;
	return ( $obj->id ne '0' )
		? '<input type="checkbox" name="sel" value="' . $obj->id . '" />'
		: DENIED_CHECK;
}
sub _display_status_mark
{
	my $self = shift;
	my $obj  = shift;
	return ($obj->stat eq '1') 
		? sb->string('[adv]') 
		: '';
}
1;
__END__
