# sb::Content::List
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==

# ======================================
package sb::Content::List;
use vars qw( $VERSION );
$VERSION = '0.11';
# 0.11 [2009/02/10] modified _latest and _selected to output correctly
# 0.10 [2009/02/05] omdified lists associated with article to use new accessor 
# 0.09 [2009/01/24] modified _latest and _selected to handle no title entry
# 0.08 [2008/11/13] modified _category_tree to display number of articles
# 0.07 [2008/05/25] fixed a bug in _amazon that url is displayed as "HASH"
# 0.06 [2007/12/07] modified _calendar to fix some bugs
# 0.05 [2007/08/12] added _init
# 0.04 [2007/08/07] changed _category_tree because "main" value has been changed
# 0.03 [2007/08/07] changed _comment and _trackback to make them faster
# 0.02 [2007/08/01] fixed a simple syntax error
# 0.01 [2007/07/31] changed _amazon to apply new methods of sb::Data::Amazon
# 0.00 [2007/07/31] divided from sb::Content
# --------------------------------------
# // defines / static variables
sub PREV_ARROW (){ '&lt;&lt;' };
sub NEXT_ARROW (){ '&gt;&gt;' };
# --------------------------------------
# // default callbacks
sub _init
{
	return (
		'archives'         => \&_archives,
		'category'         => \&_category,
		'link'             => \&_link,
		'recent_comment'   => \&_comment,
		'recent_trackback' => \&_trackback,
		'latest_entry'     => \&_latest,
		'selected_entry'   => \&_selected,
		'profile'          => \&_profile,
		'calendar'         => \&_calendar,
		'amazon'           => \&_amazon,
	);
}
# --------------------------------------
# // define callbacks
sub _archives
{
	my $cms = shift;
	my %var = @_;
	my @linklist = ('<ul>');
	my @array = sort { $b <=> $a } keys(%{$var{'entryinfo'}{'monthly'}});
	foreach my $list ( @array )
	{
		my $count = $var{'entryinfo'}{'monthly'}{$list}{'count'};
		my $name  = $var{'entryinfo'}{'monthly'}{$list}{'name'};
		my $url   = _monthly_link($list,$var{'conf'});
		push(@linklist,qq|<li><a href="$url">$name</a> ($count)</li>|);
	}
	push(@linklist,'</ul>');
	$cms->num(0);
	$cms->tag('archives_list'=>join("\n",@linklist));
	return (@linklist > 2) ? 1 : 0;
}
sub _category
{
	my $cms = shift;
	my %var = @_;
	my ($all_list,$sub_list);
	my $cat = undef;
	$all_list = _category_tree(
		'cat' => $var{'sortedcat'},
		'num' => $var{'entryinfo'}{'category'},
	);
	$cat = $var{'cat'}->{$var{'id'}} if ($var{'mode'} eq 'cat');
	if ($var{'mode'} eq 'ent' and $var{'entry'}->[0])
	{
		my $cat_id = $var{'entry'}->[0]->cat;
		$cat = $var{'cat'}->{$cat_id} if ($cat_id ne '');
	}
	if ($cat)
	{
		$sub_list = _category_tree(
			'cat'    => $var{'sortedcat'},
			'branch' => $cat->id,
			'num'    => $var{'entryinfo'}{'category'},
		);
	}
	$cms->num(0);
	$cms->tag('category_list'    => ($all_list) ? '<ul>' . $all_list . '</ul>' : '');
	$cms->tag('subcategory_list' => ($sub_list) ? '<ul>' . $sub_list . '</ul>' : '');
	return ($all_list or $sub_list) ? 1 : 0;
}
sub _comment
{
	my $cms = shift;
	my %var = @_;
	my $num = 0;
	my @linklist = ('<ul>');
	my @array = sb::Data->load('Message',
		'sort' => 'date',
		'cond' => {'stat'=>1},
		'order'=> 1,
		'num'  => $var{'conf'}->value('conf_com_disp'),
	);
	foreach my $list ( @array )
	{
		my $entry = sb::Data->load('Entry','id'=>$list->eid);
		my $url  = $entry->permalink(
			'cat'=>$var{'cat'},
			'mode'=>'com',
			'type'=>$var{'conf'}->value('conf_entry_archive')
		);
		my $subj = $entry->subj_for_list();
		my $date = sb::Time->format(
			'time'=>$list->date,
			'form'=>$var{'conf'}->value('conf_dateinlist'),
			'zone'=>$var{'conf'}->value('conf_timezone')
		);
		my $arrow = $var{'lang'}->string('=&gt;');
		my $author = $list->auth;
		push(@linklist,qq|<li>$subj<br />$arrow <a href="$url">$author$date</a></li>|);
	}
	push(@linklist,'</ul>');
	$cms->num(0);
	$cms->tag('recent_comment_list'=>join("\n",@linklist));
	return (@linklist > 2) ? 1 : 0;
}
sub _trackback
{
	my $cms = shift;
	my %var = @_;
	my $num = 0;
	my @linklist = ('<ul>');
	my @array = sb::Data->load('Trackback',
		'sort' => 'date',
		'cond' => {'stat'=>1},
		'order'=> 1,
		'num'  => $var{'conf'}->value('conf_tb_disp'),
	);
	foreach my $list ( @array )
	{
		my $entry = sb::Data->load('Entry','id'=>$list->eid);
		my $url = $entry->permalink(
			'cat'=>$var{'cat'},
			'mode'=>'tb',
			'type'=>$var{'conf'}->value('conf_entry_archive')
		);
		my $subj = $entry->subj_for_list();
		my $date = sb::Time->format(
			'time'=>$list->date,
			'form'=>$var{'conf'}->value('conf_dateinlist'),
			'zone'=>$var{'conf'}->value('conf_timezone')
		);
		my $arrow = $var{'lang'}->string('=&gt;');
		my $name = $list->name;
		push(@linklist,qq|<li>$subj<br />$arrow <a href="$url">$name$date</a></li>|);
	}
	push(@linklist,'</ul>');
	$cms->num(0);
	$cms->tag('recent_trackback_list'=>join("\n",@linklist));
	return (@linklist > 2) ? 1 : 0;
}
sub _latest
{
	my $cms = shift;
	my %var = @_;
	my @linklist = ('<ul>');
	foreach my $list ( @{$var{'entryinfo'}{'latest'}} )
	{
		my $subj = $list->subj_for_list();
		my $url = $list->permalink(
			'cat'=>$var{'cat'},
			'type'=>($var{'mode'} eq 'mob') ? 'Mobile' : $var{'conf'}->value('conf_entry_archive')
		);
		my $date = sb::Time->format(
			'time'=>$list->date,
			'form'=>$var{'conf'}->value('conf_dateinlist'),
			'zone'=>$var{'conf'}->value('conf_timezone')
		);
		push(@linklist,qq|<li><a href="$url">$subj</a>$date</li>|);
	}
	push(@linklist,'</ul>');
	$cms->num(0);
	$cms->tag('latest_entry_list'=>join("\n",@linklist));
	return (@linklist > 2) ? 1 : 0;
}
sub _selected
{
	my $cms = shift;
	my %var = @_;
	my @linklist = ('<ul>');
	my @entries = @{$var{'entry'}};
	if ($var{'mode'} eq 'ent')
	{
		my $neighbor = $var{'entryinfo'}->{'neighbor'};
		unshift(@entries,$neighbor->{'next'}) if ($neighbor->{'next'});
		push(@entries,$neighbor->{'prev'}) if ($neighbor->{'prev'});
	}
	foreach my $list ( @entries )
	{
		my $subj = $list->subj_for_list();
		my $url = $list->permalink(
			'cat'=>$var{'cat'},
			'type'=>($var{'mode'} eq 'mob') ? 'Mobile' : $var{'conf'}->value('conf_entry_archive')
		);
		my $date = sb::Time->format(
			'time'=>$list->date,
			'form'=>$var{'conf'}->value('conf_dateinlist'),
			'zone'=>$var{'conf'}->value('conf_timezone')
		);
		push(@linklist,qq|<li><a href="$url">$subj</a>$date</li>|);
	}
	push(@linklist,'</ul>');
	$cms->num(0);
	$cms->tag('selected_entry_list'=>join("\n",@linklist));
	return (@linklist > 2) ? 1 : 0;
}
sub _link
{
	my $cms = shift;
	my %var = @_;
	my @linklist = ('<ul>');
	my @array = sb::Data->load('Link','sort'=>'order','cond'=>{'disp'=>0},'order'=>1);
	my %group = ();
	foreach my $list ( @array )
	{
		$group{$list->id} = [] if ($list->is_group);
	}
	foreach my $list ( @array )
	{
		next if ($list->type !~ /^\d+$/);
		next if ($list->url eq '');
		next if (ref($group{$list->type}) ne 'ARRAY');
		push(@{$group{$list->type}},$list);
	}
	foreach my $list (@array)
	{
		next if ($list->type =~ /^\d+$/);
		my $title  = ($list->text ne '')   ? ' title="'  . $list->text  . '"'  : '';
		my $target = ($list->target ne '') ? ' target="' . $list->target . '"' : '';
		my $url    = $list->url;
		my $name   = $list->name;
		if ($list->is_group and @{$group{$list->id}})
		{
			push(@linklist,qq|<li><span$title>$name</span><ul>|);
			foreach my $lnk (@{$group{$list->id}})
			{
				my $chd_title  = ($lnk->text ne '')   ? ' title="'  . $lnk->text  . '"'  : '';
				my $chd_target = ($lnk->target ne '') ? ' target="' . $lnk->target . '"' : '';
				my $chd_url    = $lnk->url;
				my $chd_name   = $lnk->name;
				push(@linklist,qq|<li><a href="$chd_url"$chd_title$chd_target>$chd_name</a></li>|);
			}
			push(@linklist,qq|</ul></li>|);
		}
		elsif ($url ne '')
		{
			push(@linklist,qq|<li><a href="$url"$title$target>$name</a></li>|);
		}
	}
	push(@linklist,'</ul>');
	$cms->num(0);
	$cms->tag('link_list'=>join("\n",@linklist));
	return (@linklist > 2) ? 1 : 0;
}
sub _profile
{
	my $cms = shift;
	my %var = @_;
	my $baseurl = $var{'conf'}->value('conf_srv_cgi') . $var{'conf'}->value('basic_sb') . '?pid=';
	my @userlist = ('<ul>');
	my @array = sort { $b->order <=> $a->order } values( %{$var{'user'}} );
	foreach my $user ( @array )
	{
		next if ($user->disp == 1);
		my $name = ($user->real ne '') ? $user->real : $user->name;
		my $url = join('',$baseurl,$user->id);
		push(@userlist,qq|<li><a href="$url">$name</a></li>|);
	}
	push(@userlist,'</ul>');
	$cms->num(0);
	$cms->tag('user_list'=>join("\n",@userlist));
	return (@userlist > 2) ? 1 : 0;
}
sub _calendar
{
	my $cms = shift;
	my %var = @_;
	my %cal = ('no1'=>'','no2'=>'','hor'=>'','ver'=>'','tab'=>'');
	my $zone = $var{'conf'}->value('conf_timezone');
	my $date = ($var{'mode'} eq 'arc') 
		? substr($var{'cond'},0,6) 
		: sb::Time->format(
			'time'=>$var{'time'},
			'form'=>'%Year%%Mon%',
			'zone'=>$zone,
		);
	my $year  = substr($date,0,4);
	my $mon   = substr($date,4,2);
	my $week  = sb::Time->get_weekday('year'=>$year,'mon'=>$mon);
	my $end   = sb::Time->get_lastday('year'=>$year,'mon'=>$mon);
	my $today = sb::Time->format(
		'time'=>$var{'time'},
		'form'=>'%Year%%Mon%%Day%',
		'zone'=>$zone,
	);
	my $cgi   = $var{'conf'}->value('conf_srv_cgi') . $var{'conf'}->value('basic_sb') . '?day=';
	my $label = sb::Time->format(
		'time'=>($var{'mode'} eq 'arc')
			? sb::Time->new('year'=>$year,'mon'=>$mon,'zone'=>$zone)->data()
			: $var{'time'},
		'form'=>$var{'conf'}->value('conf_archivelist'),
		'lang'=>$var{'conf'}->value('conf_time_lang'),
		'zone'=>$zone,
	);
	my $prev  = ($mon != 1)  ? $year . _pad0($mon - 1) : ($year - 1) . '12';
	my $next  = ($mon != 12) ? $year . _pad0($mon + 1) : ($year + 1) . '01';
	$prev = '<a href="' . _monthly_link($prev,$var{'conf'}) . '">' . PREV_ARROW . '</a>';
	$next = '<a href="' . _monthly_link($next,$var{'conf'}) . '">' . NEXT_ARROW . '</a>';
	{ # preparation
		$cal{'no1'}  = '<table border="0" cellspacing="0" cellpadding="0" class="calendar">' . "\n";
		$cal{'no2'}  = '<table border="0" cellspacing="0" cellpadding="0" class="calendar">' . "\n";
		$cal{'no2'} .= '<tr>' . "\n" . '<td colspan="7" class="calendar_month">' . $prev . ' ';
		$cal{'no2'} .= $label . ' ' . $next . '</td>' . "\n" . '</tr>' . "\n";
		$cal{'no2'} .= '<tr><td class="weekday">Sun</td>';
		$cal{'no2'} .= '<td class="weekday">Mon</td>';
		$cal{'no2'} .= '<td class="weekday">Tue</td>';
		$cal{'no2'} .= '<td class="weekday">Wed</td>';
		$cal{'no2'} .= '<td class="weekday">Thu</td>';
		$cal{'no2'} .= '<td class="weekday">Fri</td>';
		$cal{'no2'} .= '<td class="weekday">Sat</td></tr>' . "\n";
		$cal{'ver'}  = '<br />' . "\n" . $mon . '<br />' . "\n" . '--<br />' . "\n";
		$cal{'hor'}  = $prev . ' <span class="calendar_month">';
		$cal{'hor'} .= $label . '</span> | ';
	}
	for (my $d=1;$d<=$end;$d++)
	{ # calendar body
		my $w = ($d - 1 + $week) % 7;
		my $check = $date . _pad0($d);
		$cal{'tab'} .= '<tr>' if ($d == 1 or ($w == 0 and $d > 1));
		$cal{'tab'} .= ('<td class="cell">&nbsp;</td>' x $w) if ($d == 1 and $w > 0);
		$cal{'tab'} .= '<td class="cell">';
		if ($var{'entryinfo'}{'daily'}{$check})
		{
			$cal{'tab'} .= '<a href="' . $cgi . $check . '">';
			$cal{'hor'} .= '<a href="' . $cgi . $check . '">';
			$cal{'ver'} .= '<a href="' . $cgi . $check . '">';
		}
		{ # date
			$cal{'tab'} .= ($check eq $today) ? '<span class="today">' . $d . '</span>' : $d;
			$cal{'hor'} .= ($check eq $today) ? '<span class="today">' . $d . '</span>' : $d;
			$cal{'ver'} .= ($check eq $today) ? '<span class="today">' . $d . '</span>' : $d;
		}
		if ($var{'entryinfo'}{'daily'}{$check})
		{
			$cal{'tab'} .= '</a>';
			$cal{'hor'} .= '</a>';
			$cal{'ver'} .= '</a>';
		}
		$cal{'tab'} .= '</td>';
		$cal{'hor'} .= ' ';
		$cal{'ver'} .= '<br />' . "\n";
		$cal{'tab'} .= ('<td class="cell">&nbsp;</td>' x (6 - $w)) if ($d == $end);
		$cal{'tab'} .= '</tr>' . "\n" if ($d == $end or $w == 6);
	}
	{ # finishing
		$cal{'no1'} .= $cal{'tab'} . '<tr><td colspan="7"><div style="text-align: center;" class="calendar_month">';
		$cal{'no1'} .= $prev . ' ' . $label . ' ' . $next . '</div></td></tr>' . "\n" . '</table>' . "\n";
		$cal{'no2'} .= $cal{'tab'} . '</table>';
		$cal{'ver'} .= '--<br />' . "\n" . $next . '<br />' . "\n";
		$cal{'ver'} .= $prev . '<br />' . "\n" . '--<br />' . "\n";
		$cal{'hor'} .= $next . "\n";
	}
	$cms->num(0);
	$cms->tag('calendar'=>$cal{'no1'});
	$cms->tag('calendar2'=>$cal{'no2'});
	$cms->tag('calendar_vertical'=>$cal{'ver'});
	$cms->tag('calendar_horizontal'=>$cal{'hor'});
	return 1;
}
sub _amazon
{ # recommended items
	my $cms = shift;
	my %var = @_;
	my @array = sb::Data->load('Amazon',
		'sort'  => 'order',
		'cond'  => {'stat'=>1},
		'order' => 1,
		'num'   => $var{'conf'}->value('conf_aws_disp'),
	);
	my $num = 0;
	foreach my $item ( @array )
	{
		$cms->num($num);
		$cms->tag('amazon_ProductName'=>$item->name);
		$cms->tag('amazon_Catalog'=>$item->group);
		$cms->tag('amazon_Creator'=>sb::Text->new($item->creator)->format(1));
		$cms->tag('amazon_ReleaseDate'=>$item->release);
		$cms->tag('amazon_Manufacturer'=>$item->maker);
		$cms->tag('amazon_ImageUrlSmall'=>$item->image_small->{'url'});
		$cms->tag('amazon_ImageUrlMedium'=>$item->image_medium->{'url'});
		$cms->tag('amazon_ImageUrlLarge'=>$item->image_large->{'url'});
		$cms->tag('amazon_Availability'=>$item->extra);
		$cms->tag('amazon_ListPrice'=>$item->list_price);
		$cms->tag('amazon_OurPrice'=>$item->offer_price);
		$cms->tag('amazon_comment'=>sb::Text->new($item->message)->format(1));
		$cms->tag('amazon_htmlcomment'=>sb::Text->new($item->message)->detitized());
		$cms->tag('amazon_url'=>$item->url);
		$cms->tag('amazon_item'=>$item->formatted_item);
		$num++;
	}
	return($num);
}
# --------------------------------------
# // private functions - utilities
sub _monthly_link
{
	my ($base,$conf) = @_;
	my $url = $conf->value('conf_srv_cgi') . $conf->value('basic_sb') . '?month=' . $base;
	if ($conf->value('conf_entry_archive') eq 'Monthly')
	{
		my $file = $conf->value('conf_dir_log') . $base . $conf->value('basic_suffix');
		$url = $conf->value('conf_srv_base') . $file if (-e $conf->value('conf_dir_base') . $file);
	}
	return $url;
}
sub _category_tree
{
	my %param = (
		'cat'    => [],
		'branch' => undef,
		'num'    => {},
		'no_num' => undef,
		@_
	);
	my $list = '';
	foreach my $cat ( @{$param{'cat'}} )
	{
		my $parent = ($cat->main == -1) ? '' : $cat->main;
		next if ($cat->get_option('list') == 1);
		next if (!defined($param{'branch'}) and $parent ne '');
		next if ( defined($param{'branch'}) and $parent ne $param{'branch'});
		my $text = $cat->get_option('sum') ? $cat->formatted_text('as_summary') : '';
		$text = ($text ne '') ? ' title="' . sb::Text->new($text)->entitized() . '"' : '';
		$list .= '<li><a href="' . $cat->cat_url . '"' . $text . '>' . $cat->name . '</a>';
		$list .= ' (' . int($param{'num'}->{$cat->id}) . ')' if (!$param{'no_num'});
		if ($cat->sub ne '')
		{
			$list .= "\n" . '<ul>';
			$list .= _category_tree(
				'cat'    => $param{'cat'},
				'branch' => $cat->id,
				'num'    => $param{'num'},
				'no_num' => $param{'no_num'},
			);
			$list .= '</ul>';
		}
		$list .= '</li>' . "\n";
	}
	return $list;
}
sub _pad0
{
	return ($_[0] < 10) 
		? '0' . $_[0] 
		: $_[0];
}
1;
__END__
